user_data = {
    "admins": [
        {
            "firstName": "I am",
            "lastName": "Awesome",
            "new": False,
            "uploadLocation": True,
            "uploadSwitchState": True,
            "uploadDeviceDetails": True,
            "email": "example@example.com",
            "emailVerified": True,
            "id": "i_am_awesome_id",
            "createdAt": "date_created",
            "updatedAt": "date_updated"
        }
    ],
    "members": [
        {
            "firstName": "You are",
            "lastName": "Awesome",
            "new": False,
            "uploadLocation": True,
            "uploadSwitchState": True,
            "uploadDeviceDetails": True,
            "email": "example@example.com",
            "emailVerified": True,
            "id": "you_are_awesome_id",
            "createdAt": "date_created",
            "updatedAt": "date_updated"
        }
    ],
    "guests": [
        {
            "firstName": "We are",
            "lastName": "Awesome",
            "new": False,
            "uploadLocation": True,
            "uploadSwitchState": True,
            "uploadDeviceDetails": True,
            "email": "example@example.com",
            "emailVerified": True,
            "id": "we_are_awesome_id",
            "createdAt": "date_created",
            "updatedAt": "date_updated"
        }
    ]
}