location_data_init = [
    {
        "name": "my_awesome_location_1",
        "uid": 1,
        "icon": "c1-tvSetup2",
        "id": "my_awesome_location_id_1",
        "sphereId": "my_awesome_sphere_id",
        "createdAt": "date_created",
        "updatedAt": "date_updated"
    },
    {
        "name": "my_awesome_location_2",
        "uid": 2,
        "icon": "c1-foodWine",
        "id": "my_awesome_location_id_2",
        "sphereId": "my_awesome_sphere_id",
        "createdAt": "date_created",
        "updatedAt": "date_updated"
    },
    {
        "name": "my_awesome_location_3",
        "uid": 3,
        "icon": "c1-bed",
        "id": "my_awesome_location_id_3",
        "sphereId": "my_awesome_sphere_id",
        "createdAt": "date_created",
        "updatedAt": "date_updated"
    }
]

location_data_removed = [
    {
        "name": "my_awesome_location_1",
        "uid": 1,
        "icon": "c1-tvSetup2",
        "id": "my_awesome_location_id_1",
        "sphereId": "my_awesome_sphere_id",
        "createdAt": "date_created",
        "updatedAt": "date_updated"
    },
    {
        "name": "my_awesome_location_3",
        "uid": 3,
        "icon": "c1-bed",
        "id": "my_awesome_location_id_3",
        "sphereId": "my_awesome_sphere_id",
        "createdAt": "date_created",
        "updatedAt": "date_updated"
    }
]

presence_data = [
    {
        "userId": "i_am_awesome_id",
        "locations": [
            "my_awesome_location_id_1",
            "my_awesome_location_id_2",
        ]
    },
    {
        "userId": "you_are_awesome_id",
        "locations": [
            "my_awesome_location_id_3",
        ]
    },
    {
        "userId": "we_are_awesome_id",
        "locations": [
            "my_awesome_location_id_2",
        ]
    }
]