"""Constants for the Crownstone cloud library."""
from typing import Final

DEFAULT_CLOUD_ADDR: Final = "https://cloud.ownstone.org/api"
CLOUD_LOGIN_SUFFIX: Final = "/users/login"

# Crownstone abilities
DIMMING_ABILITY: Final = "dimming"
SWITCHCRAFT_ABILITY: Final = "switchcraft"
TAP_TO_TOGGLE_ABILITY: Final = "tapToToggle"
